package hok.model;

import hok.ui.wizard.WizardView;

import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.swt.widgets.TableItem;

/**
 * The ConstraintCellModifier modifies the values of the constraint model. It Evaluates the values
 * of the celleditors attached to each column.
 * @author holger
 *
 */
public class ConstraintCellModifier implements ICellModifier {

	private WizardView view;

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ICellModifier#canModify(java.lang.Object, java.lang.String)
	 */
	public boolean canModify(Object element, String property) {

		// every column can be modified.
		return true;
	}

	/**
	 * The constructor. It sets the view.
	 * @param view the view to set.
	 */
	public ConstraintCellModifier(WizardView view) {
		super();
		this.view = view;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ICellModifier#getValue(java.lang.Object, java.lang.String)
	 */
	public Object getValue(Object element, String property) {
		// Find the index of the column
		int columnIndex = view.getColumnNamesAsList().indexOf(property);

		Object result = null;
		ConstraintRow constraint = (ConstraintRow) element;

		switch (columnIndex) {
		case 0: // Selected?
			result = new Boolean(constraint.isSelected());
			break;
		case 1: // Predicate
			result = constraint.getPredicateConstraint();
			break;
		case 2: // Concept
			result = constraint.getConceptConstraint();
			break;
		default:
			result = "";
		}
		return result;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ICellModifier#modify(java.lang.Object, java.lang.String, java.lang.Object)
	 */
	public void modify(Object element, String property, Object value) {
		// Find the index of the column
		int columnIndex = view.getColumnNamesAsList().indexOf(property);

		TableItem item = (TableItem) element;
		ConstraintRow constraint = (ConstraintRow) item.getData();
		
		switch (columnIndex) {
			case 0: // COMPLETED_COLUMN 
				constraint.setSelected(((Boolean) value).booleanValue());
				break;
			case 1:
				if(value instanceof TreeElement) {
					TreeElement selectedPredicateElement = (TreeElement) value;
					constraint.setPredicateConstraint(selectedPredicateElement.getElement());
				}
				break;
			case 2:
				if(value instanceof TreeElement) {
					TreeElement selectedConceptElement = (TreeElement) value;
					constraint.setConceptConstraint(selectedConceptElement.getElement());
				}
				break;
			default:
				
		}
		view.getConstraintmodel().constraintChanged(constraint);

	}

}
