package hok.model;

import hok.ui.wizard.WizardView;

import org.eclipse.jface.viewers.TableViewer;

/**
 * An Element of the {@link ConstraintModel} that is used as the model for the
 * {@link TableViewer} of the {@link WizardView}. It is a simple JavaBean
 * representing a row in the table and holding the values for each column of a
 * row.
 * 
 * @author holger
 * 
 */
public class ConstraintRow {

	private boolean selected;

	private String predicateConstraint;

	private String conceptConstraint;

	/**
	 * Returns the value of the concept constraint the user has set.
	 * 
	 * @return the string containing the value of the concept constraint.
	 */
	public String getConceptConstraint() {
		return conceptConstraint;
	}

	/**
	 * Sets a new value of the concept constraint.
	 * 
	 * @param conceptConstraint
	 *            the new value of the concept constraint to set.
	 */
	public void setConceptConstraint(String conceptConstraint) {
		this.conceptConstraint = conceptConstraint;
	}

	/**
	 * Returns the value of the predicate constraint the user has set.
	 * 
	 * @return the string containing the value of the predicate constraint.
	 */
	public String getPredicateConstraint() {
		return predicateConstraint;
	}

	/**
	 * Sets a new value of the predicate constraint.
	 * 
	 * @param predicateConstraint
	 *            the new value of the concept constraint to set.
	 */
	public void setPredicateConstraint(String predicateConstraint) {
		this.predicateConstraint = predicateConstraint;
	}

	/**
	 * Returns whether a row is selected.
	 * 
	 * @return True if the row is selected by the user, false otherwise.
	 */
	public boolean isSelected() {
		return selected;
	}

	/**
	 * Sets the selection state of the row.
	 * 
	 * @param selected
	 *            the boolean that indicates the selection state of the row.
	 */
	public void setSelected(boolean selected) {
		this.selected = selected;
	}

}
