package hok.model;

import java.util.Set;

/**
 * Interface that provides methods for managing a tree node. 
 * 
 * @author holger
 * 
 */
public interface ITreeNode {

	/**
	 * Returns the children of a parent element.
	 * 
	 * @param parentElement
	 *            the parent element.
	 * @return a list of ITreeNode elements.
	 */
	public Set<ITreeNode> getChildren(ITreeNode parentElement);

	/**
	 * Returns the parent of the current element.
	 * 
	 * @return the parent element.
	 */
	public ITreeNode getParent();

	/**
	 * Checks if the current element has children.
	 * 
	 * @return true if the element has children, false otherwise.
	 */
	public boolean hasChildren();

	/**
	 * Adds a child to the tree model.
	 * 
	 * @param child
	 *            the child to add.
	 */
	public void addChild(ITreeNode child);

	/**
	 * Removes a child from the model.
	 * 
	 * @param child
	 *            the child to remove.
	 */
	public void removeChild(ITreeNode child);

}
