package hok.model;

import hok.ui.wizard.ResultDialog;

import org.eclipse.jface.viewers.TableViewer;

/**
 * An element of the {@link PatternModel} that is used as the model for the
 * {@link TableViewer} of the {@link ResultDialog}. It is a simple JavaBean
 * representing a row in the table and holding the values for each column of a
 * row.
 * 
 * @author holger
 * 
 */
public class PatternElement {

	private String name;

	private String description;

	/**
	 * Returns the description of the design pattern.
	 * 
	 * @return the description of the design pattern.
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * Sets the description of the design pattern.
	 * 
	 * @param description
	 *            the string describing the design pattern.
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * Returns the name of the design pattern.
	 * 
	 * @return the string naming the design pattern.
	 */
	public String getName() {
		return name;
	}

	/**
	 * Sets the name of the design pattern.
	 * 
	 * @param name
	 *            the string naming the design pattern.
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * A design pattern is equal another design pattern if the name of both
	 * patterns is equal.
	 */
	@Override
	public boolean equals(Object obj) {
		if (obj instanceof PatternElement) {
			PatternElement elem = (PatternElement) obj;
			return getName().equals(elem.getName());
		}
		return super.equals(obj);
	}

}
