package hok.model;

/**
 * The {@link TreeElement} is a JavaBean representing an owl individual or owl
 * object property. It stores the identifier, the label and the comment of the
 * owl element.
 * 
 * @author holger
 * 
 */
public class TreeElement extends ATreeModel implements Comparable<TreeElement> {

	private String label;

	private String comment;

	private String element;

	/**
	 * Returns the URI of the owl element.
	 * 
	 * @return the String representation of the owl URI.
	 */
	public String getElement() {
		return element;
	}

	/**
	 * Sets the URI of the owl element.
	 * 
	 * @param element
	 *            the String representation of the owl URI to set.
	 */
	public void setElement(String element) {
		this.element = element;
	}

	/**
	 * Constructor. Sets the parent {@link ITreeNode} element if there is one,
	 * or null if this is the root element.
	 * 
	 * @param parent
	 *            the parent element to set.
	 */
	public TreeElement(ITreeNode parent) {
		super(parent);
	}

	/**
	 * Returns the rdfs:Comment of the owl element.
	 * 
	 * @return the String representation of the rdfs:Comment to set.
	 */
	public String getComment() {
		return comment;
	}

	/**
	 * Sets the rdfs:Comment of the owl element.
	 * 
	 * @param comment
	 *            the String representation of the rdfs:Comment to set.
	 */
	public void setComment(String comment) {
		this.comment = comment;
	}

	/**
	 * Returns the rdfs:Label of the owl element.
	 * 
	 * @return the String representation of the rdfs:Label of the owl element.
	 */
	public String getLabel() {
		return label;
	}

	/**
	 * Sets the rdfs:Label of the owl element.
	 * 
	 * @param label
	 *            the String representation of the rdfs:Label to set.
	 */
	public void setLabel(String label) {
		this.label = label;
	}

	/**
	 * Compares the element attribute of two TreeElement objects.
	 * 
	 * @param o
	 *            the object to compare.
	 */
	public int compareTo(TreeElement o) {
		return String.CASE_INSENSITIVE_ORDER.compare(this.element, o.element);
	}

}
