package hok.test;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import hok.model.ConstraintRow;
import hok.model.ITreeNode;
import hok.model.ModelMediator;
import hok.model.OntElementNotFoundException;
import hok.model.PatternModel;
import hok.model.QueryCreationFailedException;
import hok.model.TreeElement;
import hok.model.TreeFactory;
import hok.ui.wizard.Activator;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;
import java.util.Set;
import java.util.Vector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import com.hp.hpl.jena.ontology.ObjectProperty;
import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.rdf.model.ModelFactory;

public class SparqlTestCase {

	private OntModel m;

	private TreeFactory factory;

	public static final String RESOURCE_PATH = "config";

	public static final String REASONER_PATH = "reasoner.execstring";

	private Log log = LogFactory.getLog(Activator.class);

	private Process p;

	@Test
	public void testGenerateSparqlAST() {

	}

	@Before
	public void setup() throws Exception {
		startReasoner();

		m = ModelFactory.createOntologyModel(OntModelSpec.OWL_MEM, null);

		factory = new TreeFactory(m);
	}

	private void startReasoner() throws IOException {

	    Properties config = new Properties();
	    
	    config.load(new FileInputStream("config.properties"));

		assertTrue(config.containsKey(REASONER_PATH));

		String reasonerPath = config.getProperty(REASONER_PATH);

		assertNotNull(reasonerPath);

		log.info("Reasoner starting.");
		p = Runtime.getRuntime().exec(reasonerPath);
		log.info("Reasoner is running...");

	}

	@Test
	public void testGenerateOntConcepts() throws OntElementNotFoundException {
		log.info("*** testGenerateOntConcepts() ***");

		assertNotNull(m);

		OntClass cls = m.createClass("ProblemConcept");
		assertNotNull(cls);
		cls.addLabel("ProblemConcept", "EN");
		cls.addComment("ProblemConcept", "EN");
		OntClass cls2 = m.createClass("CLS2");
		cls2.addLabel("CLS2", "EN");
		cls2.addComment("CLS2", "EN");
		cls2.setSuperClass(cls);

		ITreeNode tmodel = factory
				.generateConceptTree(TreeFactory.ROOT_CONCEPT);

		assertNotNull(tmodel);

		assertNotNull(tmodel.getChildren(tmodel));

		Set<ITreeNode> tlist = tmodel.getChildren(tmodel);
		for (ITreeNode element : tlist) {
			log.info(((TreeElement) element).getLabel());
		}

		assertTrue(tmodel.getChildren(tmodel) instanceof Set);

	}

	@Test
	public void testGenerateOntProperties() throws OntElementNotFoundException {
		log.info("*** testGenerateOntProperties() ***");

		assertNotNull(m);

		ObjectProperty opr = m.createObjectProperty("constrains");
		opr.addLabel("constrains", "EN");
		ObjectProperty opr2 = m.createObjectProperty("subObjectProperty");
		opr2.addLabel("subObjectProperty", "EN");
		opr2.setSuperProperty(opr);

		ITreeNode tmodel = factory
				.generatePredicateTree(TreeFactory.ROOT_PREDICATE);

		assertNotNull(tmodel);

		assertNotNull(tmodel.getChildren(tmodel));

		Set<ITreeNode> tlist = tmodel.getChildren(tmodel);
		for (ITreeNode element : tlist) {
			System.out.println(((TreeElement) element).getLabel());
		}

		assertTrue(tmodel.getChildren(tmodel) instanceof Set);

	}

	@Test
	public void testModelMediator() throws QueryCreationFailedException,
			OntElementNotFoundException {
		ModelMediator modelMediator = new ModelMediator();
		assertNotNull(modelMediator);

		PatternModel pModel = modelMediator
				.queryModel(new Vector<ConstraintRow>());

		assertNotNull(pModel);

	}

	@Test(expected = QueryCreationFailedException.class)
	public void testModelMediatorException()
			throws QueryCreationFailedException, OntElementNotFoundException {
		new ModelMediator().queryModel(new Vector<ConstraintRow>());
	}
	
	@After
	public void shutdown() {
		if (p != null) {
			p.destroy();
			log.info("Reasoner stopped!");
		}
	}

}
