package hok.ui.wizard;

import java.io.IOException;
import java.util.ResourceBundle;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * The activator class controls the plug-in life cycle
 */
public class Activator extends AbstractUIPlugin {

	// The plug-in ID
	public static final String PLUGIN_ID = "RCPDPWizard";

	public static final String RESOURCE_PATH = "config";

	public static final String REASONER_PATH = "reasoner.execstring";

	private Log log = LogFactory.getLog(Activator.class);

	// The shared instance
	private static Activator plugin;

	private Process p;

	/**
	 * The constructor
	 */
	public Activator() {
		plugin = this;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		startReasoner();
	}

	private void startReasoner() {

		ResourceBundle config = ResourceBundle.getBundle(RESOURCE_PATH);
		if (config == null) {
			log.error("Config file " + RESOURCE_PATH + " not found!");
		} else {
			if (!config.containsKey(REASONER_PATH)) {
				log
						.error("Error in config.properties. The path to the reasoner is not set!");
			}
			String reasonerPath = config.getString(REASONER_PATH);

			if (reasonerPath == null) {
				log
						.error("Error starting the Reasoner! Path wrong or not executable.");
			} else {

				try {

					log.info("Reasoner starting.");
					p = Runtime.getRuntime().exec(reasonerPath);
					log.info("Reasoner is running...");

				} catch (IOException e) {
					log.error(e.getMessage());
					e.printStackTrace();
				}
			}

		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
		// this.execReasoner.shutdown();
		p.destroy();
		log.info("Reasoner has shutdown.");
	}

	/**
	 * Returns the shared instance
	 * 
	 * @return the shared instance
	 */
	public static Activator getDefault() {
		return plugin;
	}

	/**
	 * Returns an image descriptor for the image file at the given plug-in
	 * relative path
	 * 
	 * @param path
	 *            the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return imageDescriptorFromPlugin(PLUGIN_ID, path);
	}
}
