package hok.ui.wizard;

import hok.model.ConstraintCellModifier;
import hok.model.ConstraintModel;
import hok.model.ConstraintRow;
import hok.model.ModelMediator;
import hok.model.OntElementNotFoundException;
import hok.model.QueryCreationFailedException;

import java.util.Arrays;
import java.util.Vector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.CheckboxCellEditor;
import org.eclipse.jface.viewers.DialogCellEditor;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.part.ViewPart;

/**
 * The WizardView is build as an Eclipse RCP ViewPart. It provides a TableViewer in 
 * which the user can constrain a design pattern problem description. 
 * @author holger
 * 
 */
/**
 * @author holger
 * 
 */
public class WizardView extends ViewPart {

	public static final String ID = "RCPDPWizard.view";

	private TableViewer viewer;

	private Label l = null;

	/**
	 * The model for the {@link TableViewer}.
	 */
	private ConstraintModel constraintmodel = new ConstraintModel();

	/**
	 * The model mediator uses the elements from the constraint model to query
	 * the ontology.
	 */
	private ModelMediator modelMediator;

	private Button button_add;

	private Button button_suggest;

	private Button button_delete;

//	private StatusLineManager statusBar;

	// For the icons
	private static ImageRegistry imageRegistry = new ImageRegistry();

	// Names of images used to represent checkboxes
	private static final String CHECKED_IMAGE = "checked.gif";

	private static final String UNCHECKED_IMAGE = "unchecked.gif";

	// Name of the icon for the add button
	private static final String ADD_IMAGE = "add.gif";

	// Name of the icon for the remove button
	private static final String REMOVE_IMAGE = "remove.gif";

	// Name of the icon for the suggest pattern button
	private static final String SUGGEST_IMAGE = "suggest.gif";

	private static final String[] COLUMN_NAMES = new String[] { " ",
			"Predicate Constraint", "Concept Constraint" };

	private static final int[] COLUMN_ALIGNMENT = new int[] { SWT.LEFT,
			SWT.LEFT, SWT.LEFT };

	private static final int[] COLUMN_WIDTH = new int[] { 30, 200, 200 };

	private Log log = LogFactory.getLog(WizardView.class);

	/**
	 * Note: An image registry owns all of the image objects registered with it,
	 * and automatically disposes of them the SWT Display is disposed.
	 */
	static {
		String iconPath = "icons/";
		imageRegistry.put(CHECKED_IMAGE, Activator.getImageDescriptor(iconPath
				+ CHECKED_IMAGE));
		imageRegistry.put(UNCHECKED_IMAGE, Activator
				.getImageDescriptor(iconPath + UNCHECKED_IMAGE));
		imageRegistry.put(ADD_IMAGE, Activator.getImageDescriptor(iconPath
				+ ADD_IMAGE));
		imageRegistry.put(REMOVE_IMAGE, Activator.getImageDescriptor(iconPath
				+ REMOVE_IMAGE));
		imageRegistry.put(SUGGEST_IMAGE, Activator.getImageDescriptor(iconPath
				+ SUGGEST_IMAGE));
	}

	/**
	 * 
	 */
	public WizardView() {
		super();
		this.modelMediator = new ModelMediator();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite parent) {

		GridData gridData = new GridData(GridData.FILL_HORIZONTAL,
				GridData.VERTICAL_ALIGN_BEGINNING, true, false);

		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		layout.horizontalSpacing = 5;
		layout.verticalSpacing = 5;
		Composite top = new Composite(parent, SWT.NONE);
		top.setLayout(layout);
		top.setLayoutData(gridData);

		createLabels(top);

		createTableViewer(top);

		createButtons(top);

		constraintmodel.addConstraint();

	}

	private void addNewConstraint() {
		constraintmodel.addConstraint();
	}

	private void removeSelectedConstraints() {

		Vector<ConstraintRow> selection = this.constraintmodel
				.getConstraints();
		for (ConstraintRow row : selection) {
			if (row.isSelected()) {
				this.constraintmodel.removeConstraint(row);
			}
		}
		viewer.getTable().setRedraw(true);
	}

	private void createLabels(Composite top) {
		GridData grData = new GridData();
		grData.horizontalSpan = 2;
		l = new Label(top, SWT.WRAP);
		l.setText("Design Pattern Wizard");
		l.setFont(JFaceResources.getFontRegistry().getBold(
				JFaceResources.HEADER_FONT));
		l.setLayoutData(grData);
		
		l = new Label(top, SWT.WRAP);
		l.setText("Please describe the design problem. \n" +
				"A problem description has the form: The solution <constrains> <something>"); 
		grData = new GridData();
		grData.horizontalSpan = 2;
		grData.verticalIndent = 5;
		l.setLayoutData(grData);
	}

	private void createTableViewer(Composite top) {

		viewer = new TableViewer(top, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL
				| SWT.FULL_SELECTION);

		viewer.setContentProvider(new ViewContentProvider());

		viewer.setInput(constraintmodel);

		ViewLabelProvider labelProvider = new ViewLabelProvider();
		viewer.setLabelProvider(labelProvider);

		viewer.setCellModifier(new ConstraintCellModifier(this));

		viewer.setColumnProperties(getColumnNames());
		createColumns(viewer.getTable());
		viewer.getTable().setHeaderVisible(true);
		viewer.getTable().setLinesVisible(false);

		CheckboxCellEditor checkeditor = new CheckboxCellEditor(viewer
				.getTable());
		DialogCellEditor celleditor = new DialogCellEditor(viewer.getTable()) {

			@Override
			protected Object openDialogBox(Control cellEditorWindow) {
				// MessageDialog.openInformation(getSite().getShell(),
				// "Not Implemented", "Not Implemented, yet.");
				ConstraintDialog dialog = new ConstraintDialog(getSite()
						.getShell(), ConstraintDialog.DialogType.PredicateType);
				dialog.setTreeModel(modelMediator.getPredicateTree());
				dialog.open();
				if (dialog.getReturnCode() == IDialogConstants.OK_ID) {
					this.setValue(dialog.getSelectedElement());
				}
				System.out.println(this.getValue());
				return this.getValue();
			}
		};

		DialogCellEditor cell2editor = new DialogCellEditor(viewer.getTable()) {

			@Override
			protected Object openDialogBox(Control cellEditorWindow) {
				// MessageDialog.openInformation(getSite().getShell(),
				// "Not Implemented", "Not Implemented, yet.");
				ConstraintDialog dialog = new ConstraintDialog(getSite()
						.getShell(), ConstraintDialog.DialogType.ConceptType);
				dialog.setTreeModel(modelMediator.getConceptTree());
				dialog.open();
				if (dialog.getReturnCode() == IDialogConstants.OK_ID) {
					this.setValue(dialog.getSelectedElement());
				}
				System.out.println(this.getValue());
				return this.getValue();
			}
		};

		viewer.setCellEditors(new CellEditor[] { checkeditor, celleditor,
				cell2editor });

		GridData gridData = new GridData(GridData.FILL_BOTH);
//		GridData gridData = new GridData();
		gridData.verticalSpan = 1;
		gridData.horizontalSpan = 2;
		gridData.minimumHeight = 50;
		gridData.heightHint = 130;
		viewer.getTable().setLayoutData(gridData);

		viewer.addSelectionChangedListener(new ISelectionChangedListener() {

			public void selectionChanged(SelectionChangedEvent event) {
				if (constraintmodel.isEmpty()) {
					button_delete.setEnabled(false);
					button_suggest.setEnabled(false);
				} else {
					button_delete.setEnabled(true);
					button_suggest.setEnabled(true);
				}

			}

		});

	}

	private void createButtons(Composite top) {

		button_add = new Button(top, SWT.WRAP);
		button_add.setText("Add Constraint");
		button_add.setImage(imageRegistry.get(ADD_IMAGE));
		button_add.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				addNewConstraint();
			}
		});

		button_delete = new Button(top, SWT.WRAP);
		button_delete.setText("Delete Selected Constraints");
		button_delete.setImage(imageRegistry.get(REMOVE_IMAGE));
		button_delete.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				removeSelectedConstraints();
			}
		});
//		GridData grData = new GridData();
//		grData.horizontalSpan = 2;
//		grData.horizontalAlignment = SWT.END;
//		button_delete.setLayoutData(grData);

		button_suggest = new Button(top, SWT.WRAP);
		button_suggest.setText("Suggest Pattern");
		button_suggest.setImage(imageRegistry.get(SUGGEST_IMAGE));
		button_suggest.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {

				ResultDialog dialog = new ResultDialog(getSite().getShell());
				try {
					Vector<ConstraintRow> constraints = constraintmodel
					.getConstraints();
					dialog.setPatternModel(modelMediator.queryModel(constraints));
				} catch (QueryCreationFailedException e1) {
					log.fatal(e1);
				} catch (OntElementNotFoundException e1) {
					// TODO Auto-generated catch block
					log.fatal(e1);
				}
				dialog.open();
			}
		});
		
		GridData grData = new GridData();
		grData.horizontalSpan = 2;
		grData.horizontalAlignment = SWT.END;
		button_suggest.setLayoutData(grData);

//		statusBar = new StatusLineManager();
//		statusBar.createControl(top);
//
//		statusBar.setMessage("Status bar");

	}

	private String[] getColumnNames() {
		return COLUMN_NAMES;
	}

	/**
	 * Returns the names of the table columns as a list.
	 * 
	 * @return a list of column names
	 */
	public java.util.List getColumnNamesAsList() {
		return Arrays.asList(COLUMN_NAMES);
	}

	private int getColumnCount() {
		return COLUMN_NAMES.length;
	}

	private int[] getColumnAlignment() {
		return COLUMN_ALIGNMENT;
	}

	private int[] getColumnWidth() {
		return COLUMN_WIDTH;
	}

	private void createColumns(Table table) {
		for (int i = 0; i < getColumnCount(); i++) {
			TableColumn column = new TableColumn(table,
					getColumnAlignment()[i], i);
			column.setText(getColumnNames()[i]);
			column.setWidth(getColumnWidth()[i]);
		}
	}

	/**
	 * The content provider class is responsible for providing objects to the
	 * view. It can wrap existing objects in adapters or simply return objects
	 * as-is. These objects may be sensitive to the current input of the view,
	 * or ignore it and always show the same content (like Task List, for
	 * example).
	 */
	class ViewContentProvider implements IStructuredContentProvider,
			IConstraintViewer {
		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
			if (newInput != null)
				((ConstraintModel) newInput).addChangeListener(this);
			if (oldInput != null)
				((ConstraintModel) oldInput).removeChangeListener(this);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
		public void dispose() {
			constraintmodel.removeChangeListener(this);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
		public Object[] getElements(Object parent) {

			return constraintmodel.getConstraints().toArray();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see hok.ui.wizard.IConstraintViewer#addConstraint(hok.model.ConstraintRow)
		 */
		public void addConstraint(ConstraintRow constraint) {
			viewer.add(constraint);

		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see hok.ui.wizard.IConstraintViewer#removeConstraint(hok.model.ConstraintRow)
		 */
		public void removeConstraint(ConstraintRow constraint) {
			viewer.remove(constraint);

		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see hok.ui.wizard.IConstraintViewer#updateConstraint(hok.model.ConstraintRow)
		 */
		public void updateConstraint(ConstraintRow constraint) {
			viewer.update(constraint, null);

		}
	}

	/**
	 * Provides the text and the icon of the table elements.
	 * 
	 * @author holger
	 * 
	 */
	class ViewLabelProvider extends LabelProvider implements
			ITableLabelProvider {

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object,
		 *      int)
		 */
		public String getColumnText(Object obj, int index) {
			String result = "";
			ConstraintRow constraint = (ConstraintRow) obj;
			switch (index) {
			case 0: // edit column
				break;
			case 1:
				result = constraint.getPredicateConstraint();
				break;
			case 2:
				result = constraint.getConceptConstraint();
				break;
			default:
				break;
			}
			return result;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object,
		 *      int)
		 */
		public Image getColumnImage(Object obj, int index) {
			return (index == 0) ? // edit column?
			getImage(((ConstraintRow) obj).isSelected())
					: null;
		}

		private Image getImage(boolean isSelected) {
			String key = isSelected ? CHECKED_IMAGE : UNCHECKED_IMAGE;
			return imageRegistry.get(key);
		}

	}

	/**
	 * Returns the model of the TableViewer. It is used by th
	 * ConstraintCellModifier to inform the model of changes.
	 * 
	 * @return the constraint model
	 */
	public ConstraintModel getConstraintmodel() {
		return constraintmodel;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	@Override
	public void setFocus() {
		// TODO Auto-generated method stub

	}

} // @jve:decl-index=0:visual-constraint="10,10,570,324"
